/*
 * Copyright (C) 2006  Chris Rose
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package com.aimedia.azjabber.config;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.gudy.azureus2.plugins.PluginConfigListener;
import org.gudy.azureus2.plugins.PluginInterface;
import org.gudy.azureus2.plugins.ui.config.BooleanParameter;
import org.gudy.azureus2.plugins.ui.config.IntParameter;
import org.gudy.azureus2.plugins.ui.config.Parameter;
import org.gudy.azureus2.plugins.ui.config.PasswordParameter;
import org.gudy.azureus2.plugins.ui.config.StringParameter;
import org.gudy.azureus2.plugins.ui.model.BasicPluginConfigModel;
import org.gudy.azureus2.plugins.ui.model.PluginConfigModel;

import com.aimedia.azjabber.AzJabberPlugin;
import com.aimedia.azjabber.StateTransition;

public class JabberConfiguration implements PluginConfigListener, IAzJabberOptionNames, IAzJabberMessageKeys {

    class SavedConnectionState extends JabberConnectionParameters {
        private boolean dirty;

        public SavedConnectionState (JabberConfiguration config) {
            super (config.jabberHost.getValue (), config.jabberPort.getValue (), config.jabberService.getValue (),
                    config.clientUsername.getValue (), config.clientPassword.getValue (), config.clientResource
                            .getValue ());

            // : start off dirty
            this.dirty = true;
        }

        public boolean isDirty () {
            return dirty;
        }

        public void setClean () {
            dirty = false;
        }

    }

    private PluginInterface                          pluginInterface;

    private PluginConfigModel                        configModel;

    private StringParameter                          jabberHost;

    private IntParameter                             jabberPort;

    private StringParameter                          jabberService;

    private StringParameter                          clientUsername;

    private StringParameter                          clientResource;

    private PasswordParameter                        clientPassword;

    private BooleanParameter                         trackAddedDownloads;

    private BooleanParameter                         trackRemovedDownloads;

    private BooleanParameter                         trackDownloadCompleted;

    private BooleanParameter                         trackDownloadStarted;

    private BooleanParameter                         loggingDetail;

    private SavedConnectionState                     lastConnectionState;

    private List<IJabberConfigurationChangeListener> listeners = Collections
                                                                       .synchronizedList (new ArrayList<IJabberConfigurationChangeListener> ());

    public JabberConfiguration () {
        this.pluginInterface = AzJabberPlugin.getPlugin ().getPluginInterface ();

        prepareConfiguration ();
    }

    public PluginConfigModel getConfigurationModel () {
        return this.configModel;
    }

    private void prepareConfiguration () {
        BasicPluginConfigModel config = pluginInterface.getUIManager ().createBasicPluginConfigModel (
                AzJabberPlugin.PLUGIN_NAME);

        jabberHost = config.addStringParameter2 (SERVICE_HOST_PARAMETER, SERVICEHOST_NAME, "talk.google.com");
        jabberPort = config.addIntParameter2 (SERVICE_PORT_PARAMETER, SERVICEPORT_NAME, 5222);
        jabberService = config.addStringParameter2 (SERVICE_NAME_PARAMETER, SERVICENAME_NAME, "gmail.com");

        config.createGroup (SERVICE_CONFIG, new Parameter[] {jabberHost, jabberPort, jabberService});
        
        clientUsername = config.addStringParameter2 (CLIENT_USERNAME_PARAMETER, CLIENT_USERNAME_NAME, "");
        clientResource = config.addStringParameter2 (CLIENT_RESOURCE_PARAMETER, CLIENT_RESOURCE_NAME, "AzJabber");
        clientPassword = config.addPasswordParameter2 (CLIENT_PASSWORD_PARAMETER, CLIENT_PASSWORD_NAME,
                PasswordParameter.ET_PLAIN, new byte[] {});
        
        config.createGroup (CLIENT_CONFIG, new Parameter[] {clientUsername, clientPassword, clientResource});

        trackDownloadStarted = config.addBooleanParameter2 (EVENTS_TORRENT_START_PARAMETER, EVENTS_TORRENT_START_NAME,
                true);
        trackDownloadCompleted = config.addBooleanParameter2 (EVENTS_TORRENT_COMPLETES_PARAMETER,
                EVENTS_TORRENT_COMPLETES_NAME, true);
        trackAddedDownloads = config.addBooleanParameter2 (EVENTS_TORRENT_ADDED_PARAMETER, EVENTS_TORRENT_ADDED_NAME,
                false);
        trackAddedDownloads.setEnabled (false);
        trackRemovedDownloads = config.addBooleanParameter2 (EVENTS_TORRENT_REMOVED_PARAMETER,
                EVENTS_TORRENT_REMOVED_NAME, false);
        trackRemovedDownloads.setEnabled (false);

        config.createGroup (EVENT_CONFIG, new Parameter[] {trackDownloadCompleted, trackDownloadStarted, trackAddedDownloads, trackRemovedDownloads});
        
        linkNotificationDispatchForEvent (trackDownloadCompleted, StateTransition.DOWNLOADING_TO_SEEDING);
        linkNotificationDispatchForEvent (trackDownloadCompleted, StateTransition.DOWNLOADING_TO_STOPPING);
        linkNotificationDispatchForEvent (trackDownloadCompleted, StateTransition.SEEDING_TO_STOPPED);
        linkNotificationDispatchForEvent (trackDownloadCompleted, StateTransition.SEEDING_TO_STOPPING);
        linkNotificationDispatchForEvent (trackDownloadCompleted, StateTransition.DOWNLOADING_TO_STOPPED);

        linkNotificationDispatchForEvent (trackDownloadStarted, StateTransition.READY_TO_DOWNLOADING);

        loggingDetail = config.addBooleanParameter2 (LOGGING_DETAILED_PARAMETER, LOGGING_DETAILED_NAME, true);
        configModel = config;

        // : check to see if this is the first run
        boolean firstRun = pluginInterface.getPluginconfig ().getPluginBooleanParameter (EVENTS_FIRST_RUN, true);
        
        /*
         * track configuration save events, so that we can ensure that we don't
         * reconnect overzealously.
         */
        lastConnectionState = new SavedConnectionState (this);

        AzJabberPlugin.getPlugin ().getPluginInterface ().getPluginconfig ().addListener (this);
    }

    private void linkNotificationDispatchForEvent (BooleanParameter param, StateTransition event) {
        /*
         * if the option is enabled, no notification is sent automatically, so
         * we have to enable these messages.
         */
        NotificationDispatcher dispatcher;
        dispatcher = new NotificationDispatcher (event);
        param.addListener (dispatcher);
        dispatcher.parameterChanged (param);
    }

    /**
     * @return the clientPassword
     */
    public byte[] getClientPassword () {
        return clientPassword.getValue ();
    }

    /**
     * @return the clientResource
     */
    public String getClientResource () {
        return clientResource.getValue ();
    }

    /**
     * @return the clientUsername
     */
    public String getClientUsername () {
        return clientUsername.getValue ();
    }

    /**
     * @return the jabberHost
     */
    public String getJabberHost () {
        return jabberHost.getValue ();
    }

    /**
     * @return the jabberPort
     */
    public int getJabberPort () {
        return jabberPort.getValue ();
    }

    /**
     * @return the jabberService
     */
    public String getJabberService () {
        return jabberService.getValue ();
    }

    /**
     * @return the loggingDetail
     */
    public boolean isLoggingDetailed () {
        return loggingDetail.getValue ();
    }

    /**
     * @return the trackAddedDownloads
     */
    public boolean isTrackingAddedDownloads () {
        return trackAddedDownloads.getValue ();
    }

    /**
     * @return the trackRemovedDownloads
     */
    public boolean isTrackingRemovedDownloads () {
        return trackRemovedDownloads.getValue ();
    }

    public boolean isConfigured () {
        return clientPassword.getValue () != null && clientUsername.getValue () != null
                && clientPassword.getValue ().length > 0 && clientUsername.getValue ().length () > 0;
    }

    public boolean isDirty () {
        // : TODO Intelligently detect dirty connection parameters.
        return this.lastConnectionState.isDirty ();
    }

    public void markClean () {
        this.lastConnectionState.setClean ();
    }

    public void configSaved () {
        SavedConnectionState currentState = new SavedConnectionState (this);
        if (!currentState.equals (lastConnectionState)) {
            // : the connection parameters have changed; replace the state with
            // this one.
            lastConnectionState = currentState;

            for (IJabberConfigurationChangeListener listener : listeners) {
                listener.jabberConfigurationChanged (lastConnectionState);
            }
        }
    }

    public void addJabberConfigurationChangeListener (IJabberConfigurationChangeListener listener) {
        listeners.add (listener);
    }

    public void removeJabberConfigurationChangeListener (IJabberConfigurationChangeListener listener) {
        listeners.remove (listener);
    }

    public JabberConnectionParameters getJabberConnectionParameters () {
        return new SavedConnectionState (this);
    }

}
